/**
 * This file is part of pppBenchmark, a program accompanying
 * the paper: Abeel, T, Van de Peer Y, Saeys Y.  2009.  Towards a gold standard for promoter prediction evaluation. Bioinformatics. 25(12):i313-i320.
    * 
 * Copyright 2009 Thomas Abeel
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *
 */
package be.abeel.pppbenchmark;

import java.text.NumberFormat;
import java.util.BitSet;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import be.abeel.io.LineIterator;

/**
 * protocol based on the work by Sonnenburg et al.
 * 
 * @author Thomas Abeel
 * 
 */
public class BinGeneProtocol extends Protocol {

	/*
	 * Returns Map<threshold,int[tp,tn,fp,fn]>;
	 */
	public Map<Double, int[]> run(String chr, LineIterator pred) throws Exception {
		System.out.println("\tRunning BinGene: " + chr);
		BitSet genesStart = new BitSet((int) (Config.chrSize.get(chr) / Config.distance) + 1);
		BitSet genesCoverage = new BitSet((int) (Config.chrSize.get(chr) / Config.distance) + 1);
		double[] scores = new double[(int) (Config.chrSize.get(chr) / Config.distance) + 1];

		SortedMap<Double, int[]> out = new TreeMap<Double, int[]>();

		for (int i = 0; i < scores.length; i++) {
			scores[i] = Double.NEGATIVE_INFINITY;
		}
		/* Load tags for the selected chromosome */
		LineIterator it = new LineIterator(Config.gene());
		for (String line : it) {
			String[] arr = line.split("\t");
			if (arr[0].equalsIgnoreCase(chr)) {
				int start = Integer.parseInt(arr[1]);
				int end = Integer.parseInt(arr[2]);
				boolean forward = arr[5].equals("+");
				for (int i = start / Config.distance; i <= end / Config.distance; i++)
					genesCoverage.set(i);
				if (forward) {
					genesStart.set(start / Config.distance);

				} else
					genesStart.set(end / Config.distance);

			}

		}
		it.close();
		/* Load predictions for the selected chromosome */

		for (String line : pred) {
			String[] arr = line.split("\t");
			if (arr[0].equalsIgnoreCase(chr)) {
				int start = Integer.parseInt(arr[1]);
				int end = Integer.parseInt(arr[2]);
				int middle = start / 2 + end / 2;
				double score = Double.parseDouble(arr[3]);
				
				if (scores[middle / Config.distance] < score)
					scores[middle / Config.distance] = score;
				
			}

		}
		pred.close();
		/* Do analysis with moving threshold */
		double range = 1 / Config.steps;
		
		for (double d = 0; d <= 1; d += range) {
			BitSet prediction = new BitSet((int) (Config.chrSize.get(chr) / Config.distance) + 1);
			for (int i = 0; i < scores.length; i++) {
				if (scores[i] >= d)
					prediction.set(i);
			}
			int tp = 0, tn = 0, fp = 0, fn = 0;
			for (int i = 0; i < scores.length; i++) {
				if (genesCoverage.get(i)) {
					if (genesStart.get(i) && prediction.get(i)) {
						tp++;
					}
					if (genesStart.get(i) && !prediction.get(i)) {
						fn++;
					}
					if (!genesStart.get(i) && prediction.get(i)) {
						fp++;
					}
					if (!genesStart.get(i) && !prediction.get(i)) {
						tn++;
					}
				}
			}
			NumberFormat nf = NumberFormat.getInstance();
			nf.setMaximumFractionDigits(2);
			out.put(d, new int[] { tp, tn, fp, fn });

		}
		return out;
	}

}
